//  2005 The Weather Channel Interactive, Inc.  All Rights Reserved.

/*
   The XMLRequest object can handle blocking and nonblocking calls.
   If the setParser method is called, it will turn the object into a nonblocking object.
   The setParser call requires the object that is passed into the method to have 2 methods:
      parseFunc();
      parseError();
   
   Basic usage:

         xmlRequest = new XMLRequest(url);
         xmlRequest.setParser(parserObject);
         xmlRequest.Get();

*/

function XMLRequest(url){
   this.url = url;

   this.xmlDoc = null;
   this.timeout = 1000;

   var timer = null;
   var parseObj = null;
   var request = null;

   this.setParser = function(TparseObj){
      parseObj = TparseObj;
   }

   this.setUrl = function(url){
      if(url){
         this.url = url;
      }
   }

   this.setTimeout = function(t){
      this.timeout = t;
   }

   this.Get = function(){
		
		if(this.url){
        	if(!parseObj){
            	return(this.Blocking());
         	}else{
            	this.NonBlocking();
         	}
      	}else{
        	return(false);
      	}
   }

   this.Blocking = function(){
      if(this.url){
         try{
            if(request){
               delete(request);
            }
            request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);

            request.overrideMimeType('text/xml');
			
			/* Track request */
            request.open('GET', this.url, false);
			
            request.send(null);
            if(request.status == 200){
               return(request.responseXML);
            }
			else{
				return (false);
			}
         }

         // we catch any errors to make sure we don't fill the javascript console with junk
         catch(e){
              return(null);
         }
      }
      return(null);
   }

   this.NonBlocking = function(){
	  if(this.url && parseObj){
         if(request){
            delete(request);
         }
         request = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);
         request.overrideMimeType('text/xml');
		 request.open('GET', this.url, true);
         
         // we attach an anonymous function to the onreadystatechange method of the request.
         // when state changes, this function gets called
         request.onreadystatechange = this.AsyncCall;
         
         timer = new oTimer(this.callTimeout);
         timer.setDelay(this.timeout);
         timer.setNote(this.url);
         timer.Start();

         // now that we have created the request, actually send it
         request.send(null);
         return(false);
      }else{
         return(false);
      }
   }

   this.AsyncCall = function(){
      var reqState = request.readyState;

      try{
         switch(reqState){
            // once we get to a state '4', we have finished the request so we move on
            case 4:
               timer.Stop();
               if(request.status == 200){
                  parseObj.parseFunc(request.responseXML);
                  delete(request);
                  return(true);
               }else{
                  parseObj.parseError(request.status);
                  delete(request);
                  return(false);
               }
               break;

            // if we manage to get a '2' for state, make sure there is a status number for the request.
            // if we didn't get anything for the status, we have an error
            case 2:
               try{
                  if(!request.status){
                  }
               }catch(e){
                  timer.Stop();
                  request.abort();
                  parseObj.parseError("bad status/state combo");
                  delete(request);
                  return(false);
               }                  
               break;

            default:
               // the catch all
         }
      }catch(e){
         debug("timer: " + e);
         timer.Stop();
         // if we didn't get an error about getting status, throw the generic error
         // we are ok with a status error since an abort will generally cause it
         var str = e.message;
         if(!str.match(/nsIXMLHttpRequest.status/i)){
            parseObj.parseError(e.message);
         }
         delete(request);
         return(false);
      }
   }

   this.abort = function(){
      if(timer){
         timer.Stop();
      }
      if(request){
         request.abort();
      }
   }

   this.callTimeout = function(){
      timer.Stop();
      if(request){
         try{
            request.abort();
            throw("my timeout");
            parseObj.parseError('timeout');
         }catch(e){
//            alert("e: " + e);
         }
      }else{
         parseObj.parseError('timeout');
      }
   }

}

